#!/usr/bin/python3

#
# Copyright 2019 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
import os
import re
import yaml

from oslo_concurrency import processutils

_PASSTHROUGH_WHITELIST_KEY = 'nova::compute::pci::passthrough'
_PCI_DEVICES_PATH = '/sys/bus/pci/devices'
_SYS_CLASS_NET_PATH = '/sys/class/net'
_DERIVED_PCI_WHITELIST_FILE = '/etc/puppet/hieradata/pci_passthrough_whitelist.json'

MAX_FUNC = 0x7
MAX_DOMAIN = 0xFFFF
MAX_BUS = 0xFF
MAX_SLOT = 0x1F
ANY = '*'
REGEX_ANY = '.*'

ADD_PF_PCI_ADDRESS = 0x1
ADD_VF_PCI_ADDRESS = 0x2
DEL_USER_CONFIG = 0x3
KEEP_USER_CONFIG = 0x4


class InvalidConfigException(ValueError):
    pass


def get_pci_field_val(
    prop: str, maxval: int, hex_value: str
) -> None:
    if prop == ANY:
        return REGEX_ANY
    try:
        v = int(prop, 16)
    except ValueError:
        raise InvalidConfigException('Invalid PCI address specififed {!r}'.format(prop))
    if v > maxval:
        raise InvalidConfigException('PCI address specififed {!r} is out of range'.format(prop))
    return hex_value % v


def get_pciaddr_dict_from_usraddr(pci_addr: str):
    """Convert PCI address in STRING format to DICT
    (this is done for uniformity in PCI address)
    """
    pci_dict = {}
    dbs, sep, func = pci_addr.partition('.')
    pci_dict['function'] = ANY
    if func:
        func = func.strip()
        pci_dict['function'] = func
    if dbs:
        dbs_fields = dbs.split(':')
        if len(dbs_fields) > 3:
            raise InvalidConfigException('Invalid PCI address specififed {!r}'.format(pci_addr))
        # If we got a partial address like ":00.", we need to turn this
        # into a domain of ANY, a bus of ANY, and a slot of 00. This code
        # allows the address,bus and/or domain to be left off
        dbs_all = [ANY] * (3 - len(dbs_fields))
        dbs_all.extend(dbs_fields)
        dbs_checked = [s.strip() or ANY for s in dbs_all]

        ''' domain, bus, slot = dbs_checked '''
        pci_dict['domain'], pci_dict['bus'], pci_dict['slot'] = dbs_checked

    pci_dict['domain'] = get_pci_field_val(pci_dict['domain'], MAX_DOMAIN, '%04x')
    pci_dict['slot'] = get_pci_field_val(pci_dict['slot'], MAX_SLOT, '%02x')
    pci_dict['bus'] = get_pci_field_val(pci_dict['bus'], MAX_BUS, '%02x')
    pci_dict['function'] = get_pci_field_val(pci_dict['function'], MAX_FUNC, '%1x')
    return pci_dict


def get_pci_regex_pattern(config_regex: str, size: int, maxval: int, hex_value: str):
    if config_regex in [ANY, REGEX_ANY]:
        config_regex = "[0-9a-fA-F]{%d}" % size

    try:
        re.compile(config_regex)
    except re.error:
        msg = "Invalid regex pattern identified %s" % config_regex
        raise InvalidConfigException(msg)
    try:
        v = int(config_regex, 16)
    except ValueError:
        return config_regex
    if v > maxval:
        msg = "Invalid pci address"
        raise InvalidConfigException(msg)
    return hex_value % v


def get_user_regex_from_addrdict(addr_dict):
    if isinstance(addr_dict, dict):
        domain_regex = get_pci_regex_pattern(addr_dict['domain'], 4, MAX_DOMAIN, '%04x')
        bus_regex = get_pci_regex_pattern(addr_dict['bus'], 2, MAX_BUS, '%02x')
        slot_regex = get_pci_regex_pattern(addr_dict['slot'], 2, MAX_SLOT, '%02x')
        function_regex = get_pci_regex_pattern(addr_dict['function'], 1, MAX_FUNC, '%1x')

        user_address_regex = '%s:%s:%s.%s' % (
           domain_regex, bus_regex, slot_regex,
           function_regex)
        return user_address_regex
    else:
        return None


def get_pci_addr_from_ifname(ifname: str):
    """Given the device name, returns the PCI address of a device
    and returns True if the address is in a physical function.
    """
    dev_path = os.path.join(_SYS_CLASS_NET_PATH, ifname, "device")
    if os.path.isdir(dev_path):
        try:
            return (os.readlink(dev_path).strip("./"))
        except OSError:
            raise
    return None


def get_sriov_configs():
    configs = []
    try:
        with open('/var/lib/os-net-config/sriov_config.yaml') as sriov_config:
            configs = yaml.safe_load(sriov_config)
    except IOError:
        configs = []
    return configs


def get_sriov_nic_partition_pfs(configs):
    pfs = []
    for config in configs:
        device_type = config.get('device_type', None)
        if device_type and 'vf' in device_type:
            device = config.get('device', None)
            if device:
                dev_name = device.get('name', None)
                if dev_name and dev_name not in pfs:
                    pfs.append(dev_name)
    return pfs


def get_sriov_non_nic_partition_pfs(configs):
    all_pfs = []
    non_nicp_pfs = []
    nicp_pfs = get_sriov_nic_partition_pfs(configs)

    for config in configs:
        device_type = config.get('device_type', None)
        if device_type and 'pf' in device_type:
            name = config.get('name', None)
            if name and name not in all_pfs:
                all_pfs.append(name)
    non_nicp_pfs = [x for x in all_pfs if x not in nicp_pfs]
    return non_nicp_pfs


def get_pci_device_info_by_ifname(pci_dir, sub_dir):
    if not os.path.isdir(os.path.join(pci_dir, sub_dir)):
        msg = f'{pci_dir} specified is not valid'
        raise InvalidConfigException(msg)

    try:
        # ids located in files inside PCI devices
        # directory are stored in hex format (0x1234 for example)
        with open(os.path.join(pci_dir, sub_dir,
                               'vendor')) as vendor_file:
            vendor = vendor_file.read().strip()
        with open(os.path.join(pci_dir, sub_dir,
                               'device')) as product_file:
            product = product_file.read().strip()
        return vendor, product
    except IOError:
        return '', ''


def get_available_vf_pci_addresses_by_ifname(pf_name, allocated_pci):
    """It gets the list of all VF's of a PF minus the VFs allocated for
    NIC Partitioning. If the PF has 10 VFs and VFs 0,1 are allocated then
    the PCI address of VFs from 2 to 9 along with the device info wout be returned
    """
    vf_pci_addresses = {}
    pci_dir = _PCI_DEVICES_PATH
    if os.path.isdir(pci_dir):
        for sub_dir in os.listdir(pci_dir):
            if sub_dir in allocated_pci:
                continue
            pci_phyfn_dir = os.path.join(pci_dir, sub_dir, 'physfn/net')
            if os.path.isdir(pci_phyfn_dir):
                phyfn_dirs = os.listdir(pci_phyfn_dir)
                for phyfn in phyfn_dirs:
                    if phyfn in pf_name:
                        if phyfn not in vf_pci_addresses:
                            vf_pci_addresses[phyfn] = [sub_dir]
                        else:
                            vf_pci_addresses[phyfn].append(sub_dir)
    return vf_pci_addresses


def get_allocated_pci_addresses(configs):
    alloc_pci_info = []
    for config in configs:
        pci = config.get('pci_address', None)
        if pci:
            alloc_pci_info.append(pci)
    return alloc_pci_info


def get_pci_passthrough_whitelist(user_config, pf, pci_addresses):
    pci_passthrough_list = []

    for pci in pci_addresses:
        pci_passthrough = dict(user_config)
        pci_passthrough['address'] = str(pci)

        # devname and address fields can't co exist
        if 'devname' in pci_passthrough:
            del pci_passthrough['devname']

        pci_passthrough_list.append(pci_passthrough)
    return pci_passthrough_list


def user_passthrough_config():
    try:
        out, err = processutils.execute(
            'hiera', '-f', 'json', '-c', '/etc/puppet/hiera.yaml',
            _PASSTHROUGH_WHITELIST_KEY
            )
        if not err:
            data = json.loads(out)
            # Check the data type of first json decode
            if isinstance(data, str):
                # Decode once again to get the list
                return (json.loads(data))
            elif isinstance(data, list):
                return data
    except processutils.ProcessExecutionError:
        raise


def match_pf_details(user_config, pf_name, is_non_nic_pf: bool):
    """Decide the action for whitelist_pci_addr, based on user config

    :param user_configs: THT param NovaPCIPassthrough
    :param pf_name: Interface/device name (str)
    :param is_non_nic_pf: Indicates whether the PF is noc-partitioned or not
    :return: Return the actions to be done, based on match criteria
    """

    # get the vendor and product id of the PF and VF
    pf_path = os.path.join(_SYS_CLASS_NET_PATH, pf_name, "device")
    vendor, vf_product = get_pci_device_info_by_ifname(pf_path, 'virtfn0')
    vendor, pf_product = get_pci_device_info_by_ifname(pf_path, '')

    if ('product_id' not in user_config or
         vf_product[-4:] == user_config['product_id'][-4:]):
        if is_non_nic_pf:
            # If NON NIC Part PF matches, then add the complete device
            return ADD_PF_PCI_ADDRESS
        else:
            """ In case of NIC Partitioning PF, add the VFs only (excluding NIC Part VFs)
                when the product id is not given or product_id matches that of VF
            """
            return ADD_VF_PCI_ADDRESS
    elif ('product_id' not in user_config or
            pf_product[-4:] == user_config['product_id'][-4:]):
        if is_non_nic_pf:
            # If product id of NON NIC Part VF matches, then add the complete device
            return ADD_PF_PCI_ADDRESS
        else:
            """ When the user_config address matches that of the NIC Partitioned PF,
                the PF must be removed from the user_config. So return the status such
                that the caller ignores this user_config if this user_config is very
                specific to the NIC Partitioned PF
            """
            return DEL_USER_CONFIG
    else:
        """ The Product ID neither belongs to VF nor PF, simply ignore matching
        """
        return KEEP_USER_CONFIG


# +------------+-----------------------+---------------------------+-------------------+
# |   USER     |     product_id        |       product_id          |  Product ID       |
# |   CONFIG   |     is VF             |       is PF               |  NOT mentioned    |
# +------------+-----------------------+---------------------------+-------------------+
# |   PCI addr |                       |                           |                   |
# |   is VF    |     Matching VF       |       INVALID             |  add only VFs     |
# |            |                       |                           |                   |
# |            |                       |                           |                   |
# +------------+-----------------------+---------------------------+-------------------+
# |            |                       |                           |                   |
# |   PCI addr |     ALL VFs of this   |       Matching PFs        | Add both PF and   |
# |   is PF    |     addr -            |      - NIC Part PFs       | available VFs     |
# |            |     NIC Part VFs      |                           |                   |
# |            |                       |                           |                   |
# +------------+-----------------------+---------------------------+-------------------+
# |            |                       |                           |                   |
# |  PCI addr  |     All matching      |       All matching        |  INVALID CASE     |
# |     not    |     VFs               |       PFs - NIC Part PF   |                   |
# |  specified |     NIC Part VFs      |                           |                   |
# |            |                       |                           |                   |
# +------------+-----------------------+---------------------------+-------------------+
def get_passthrough_config(user_config, pf_name,
  allocated_pci, is_non_nic_pf: bool):
    """Handle all variations of user specifid pci addr format

    Arrive at the address fields of the whitelist.  Check the address fields of
    the pci.passthrough_whitelist configuration option, validating the address fields.

    :param user_configs: THT param NovaPCIPassthrough
    :param pf_name: Interface/device name (str)
    :param allocated_pci: List of VFs (for nic-partitioned PF), which are used by host
    :param is_non_nic_pf: Indicates whether the PF is non-partitioned or not
    :return: pci_passthrough: Derived config list
    :return del_user_config: Flag to state if the user_config is to be deleted or not

    Example format for user_config:
    | [pci] in standard string format
    | passthrough_whitelist = {"address":"*:0a:00.*",
                          "physical_network":"physnet1"}
    | passthrough_whitelist = {"address": {"domain": ".*",
                                       "bus": "02",
                                       "slot": "01",
                                       "function": "[0-2]"},
                            "physical_network":"net1"}
    """
    sel_addr = []
    pci_passthrough = []
    del_user_config = False

    """ Get the regex of the address fields """
    if 'address' in user_config:
        if isinstance(user_config['address'], dict):
            addr_dict = user_config['address']
        elif isinstance(user_config['address'], str):
            addr_dict = get_pciaddr_dict_from_usraddr(user_config['address'])
        user_address_pattern = get_user_regex_from_addrdict(addr_dict)
    else:
        user_address_pattern = ("[0-9a-fA-F]{4}:[0-9a-fA-F]{2}:"
                                    "[0-9a-fA-F]{2}.[0-7]")

    """ If address mentioned in user_config is PF, then get all VFs belonging to that PF
    """
    available_vfs = get_available_vf_pci_addresses_by_ifname(pf_name, allocated_pci)

    # get the pci address of the PF
    parent_pci_address = get_pci_addr_from_ifname(pf_name)
    user_address_regex = re.compile(user_address_pattern)

    # Match the user_config address with the PF's address
    if user_address_regex.match(parent_pci_address):
        match_id = match_pf_details(user_config, pf_name, is_non_nic_pf)
        """ if the address matches and there is no mismatch in
        product id's of PF, then add the PF's provided its not a
        NIC Partitioning PF
        """
        if match_id == ADD_PF_PCI_ADDRESS:
            sel_addr.append(parent_pci_address)
        elif match_id == ADD_VF_PCI_ADDRESS:
            if pf_name in available_vfs:
                for vf_addr in available_vfs[pf_name]:
                    sel_addr.append(vf_addr)
            if not sel_addr:
                del_user_config = True
        elif match_id == DEL_USER_CONFIG:
            del_user_config = True

        """ Match the user_config address with the VF's address
        A Regex of addresses could match both the VF and PF's.
        If it matches the PF, all available VFs would be included anyway,
        so it will be inclusive even if the address matches the VFs of the same PF
        Also if product id is specified, it must match that of the VF's.
        If product id is not mentioned in user_config, its assumed to have
        matched and is left to address matching for derived configuration.
        If Address is not mentioned, then all available VF's (excluding NIC partitioned VFs)
        with the matching product id shall be added to the derived configuration.
        """
    else:
        pf_path = os.path.join(_SYS_CLASS_NET_PATH, pf_name, "device")
        vendor, vf_product = get_pci_device_info_by_ifname(pf_path, 'virtfn0')
        if (('product_id' not in user_config or
            user_config['product_id'][-4:] == vf_product[-4:]) and
            pf_name in available_vfs):
            for vf_addr in available_vfs[pf_name]:
                user_address_regex = re.compile(user_address_pattern)
                if user_address_regex.match(vf_addr):
                    sel_addr.append(vf_addr)
            if not sel_addr:
                for vf_addr in allocated_pci:
                    if user_address_regex.match(vf_addr):
                        """ When the user_config address matches that of the NIC Partitioned VF,
                        the VF must be removed from the user_config. So return the status such
                        that the caller ignores this user_config if this user_config is very
                        specific to the NIC Partitioned VF. If the user_config resulted in the
                        derivations of other configurations, the derived configuration shall
                        replace the original user_config.
                        """
                        del_user_config = True

    if sel_addr:
        pci_passthrough = get_pci_passthrough_whitelist(
            user_config, pf_name, sel_addr)
    return pci_passthrough, del_user_config


def get_passthrough_config_for_all_pf(user_config,
                                  system_configs,
                                  allocated_pci):
    derived_config = []
    nic_partition_pfs = get_sriov_nic_partition_pfs(system_configs)
    non_nic_partition_pfs = get_sriov_non_nic_partition_pfs(system_configs)
    del_user_config = False

    """
    For each user_config, do the matching for NIC Partitioning PFs/VFs
    """
    for pf in nic_partition_pfs:
        passthrough_tmp, status = get_passthrough_config(
            user_config, pf, allocated_pci, False)
        del_user_config = del_user_config or status
        if passthrough_tmp:
            derived_config.extend(passthrough_tmp)

    """
    If there is no config added from NIC Part nics, then skip parsing the
    NON NIC Partitioned PFs.
    """
    if (derived_config or del_user_config):
        for pf in non_nic_partition_pfs:
            passthrough_tmp, status = get_passthrough_config(
                user_config, pf, allocated_pci, True)
            derived_config.extend(passthrough_tmp)

    if derived_config:
        return derived_config, False
    else:
        return derived_config, del_user_config


def get_pf_name_from_phy_network(physical_network):
    try:
        out, err = processutils.execute(
            'hiera', '-c', '/etc/puppet/hiera.yaml',
            'neutron::agents::ml2::sriov::physical_device_mappings')
        if not err:
            phys_dev_mappings = json.loads(out)
            ''' Check the data type of first json decode '''
            if not isinstance(phys_dev_mappings, list):
                msg = f'ml2::sriov::physical_device_mappings specified is not a list {phys_dev_mappings}'
                raise InvalidConfigException(msg)

            for phy_dev_map in phys_dev_mappings:
                net_name, nic_name = phy_dev_map.split(':')
                if net_name == physical_network:
                    return nic_name
            return None
    except processutils.ProcessExecutionError:
        raise


def generate_combined_configuration(user_configs, system_configs):
    """Derived configuration = user_config - system_configs

    Identify the user_defined configuration that overlaps with the
    NIC Partitioned VFs and remove those VFs from the derived configuration
    In case of no overlap, the user defined configuration shall be used
    as it is.
    :param user_configs: THT param NovaPCIPassthrough
    :param system_configs: Derived from sriov-mapping.yaml
    :return user_config_copy: Any non-nic partinioned cfg will be returned in this list
    :return derived_config: All nic partinioned cfg will be returned after derivation in this list
    """

    user_config_copy = []
    derived_config = []

    allocated_pci = get_allocated_pci_addresses(system_configs)
    nic_partition_pfs = get_sriov_nic_partition_pfs(system_configs)

    for user_config in user_configs:
        if 'address' in user_config and 'devname' in user_config:
            msg = f"Both devname and address can't be present in {_PASSTHROUGH_WHITELIST_KEY}"
            raise InvalidConfigException(msg)

        keys = ['address', 'product_id', 'devname']
        if not any(k in user_config for k in keys):
            # address or product_id or devname not present in user_config
            pf = get_pf_name_from_phy_network(user_config['physical_network'])
            user_config['address'] = get_pci_addr_from_ifname(pf)

        if 'devname' in user_config:
            if user_config['devname'] in nic_partition_pfs:
                user_config['address'] = get_pci_addr_from_ifname(user_config['devname'])
                del user_config['devname']
            else:
                user_config_copy.append(user_config)
                continue
        if ('address' in user_config or
           'product_id' in user_config):
            passthrough_tmp, del_user_config = get_passthrough_config_for_all_pf(
                user_config, system_configs, allocated_pci)

            """ If del_user_config is set, do not add to derived_config or
                user_config_copy
            """
            if not del_user_config:
                if passthrough_tmp:
                    derived_config.extend(passthrough_tmp)
                else:
                    user_config_copy.append(user_config)
        else:
            user_config_copy.append(user_config)
    return (user_config_copy, derived_config)


if __name__ == "__main__":
    pci_passthrough = {}
    pci_file_path = _DERIVED_PCI_WHITELIST_FILE
    system_configs = get_sriov_configs()
    user_configs = user_passthrough_config()

    # Check if user config list is valid
    if not isinstance(user_configs, list):
        raise InvalidConfigException('user_config specified is not a list {!r}'.format(user_configs))

    user_config_copy, derived = generate_combined_configuration(
        user_configs, system_configs)

    """ If the derivation does not bring in any changes, the user_config_copy list
        and user_configs shall be same. The pci_passthrough_whitelist.json
        shall be updated only when there is a change needed due to NIC Partition
    """
    if user_config_copy != user_configs:
        pci_passthrough[_PASSTHROUGH_WHITELIST_KEY] = (user_config_copy +
                                                       derived)
        with open(pci_file_path, 'w') as pci_file:
            json.dump(pci_passthrough, pci_file)
    else:
        print("user_configs is good, nothing to be modified")



#!/usr/bin/python3
#
# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import logging
import os
import pwd
import selinux
import stat
import sys

debug = os.getenv('__OS_DEBUG', 'false')

if debug.lower() == 'true':
    loglevel = logging.DEBUG
else:
    loglevel = logging.INFO

logging.basicConfig(stream=sys.stdout, level=loglevel)
LOG = logging.getLogger('nova_statedir')


class PathManager(object):
    """Helper class to manipulate ownership of a given path"""
    def __init__(self, path):
        self.path = path
        self.uid = None
        self.gid = None
        self.is_dir = None
        self.secontext = None
        self._update()

    def _update(self):
        try:
            statinfo = os.stat(self.path)
            self.is_dir = stat.S_ISDIR(statinfo.st_mode)
            self.uid = statinfo.st_uid
            self.gid = statinfo.st_gid
            self.secontext = selinux.lgetfilecon(self.path)[1]
        except Exception:
            LOG.exception('Could not update metadata for %s', self.path)
            raise

    def __str__(self):
        return "uid: {} gid: {} path: {}{}".format(
            self.uid,
            self.gid,
            self.path,
            '/' if self.is_dir else ''
        )

    def has_owner(self, uid, gid):
        return self.uid == uid and self.gid == gid

    def has_either(self, uid, gid):
        return self.uid == uid or self.gid == gid

    def chown(self, uid, gid):
        target_uid = -1
        target_gid = -1
        if self.uid != uid:
            target_uid = uid
        if self.gid != gid:
            target_gid = gid
        if (target_uid, target_gid) != (-1, -1):
            LOG.info('Changing ownership of %s from %d:%d to %d:%d',
                     self.path,
                     self.uid,
                     self.gid,
                     self.uid if target_uid == -1 else target_uid,
                     self.gid if target_gid == -1 else target_gid)
            try:
                os.chown(self.path, target_uid, target_gid)
                self._update()
            except Exception:
                LOG.exception('Could not change ownership of %s: ',
                              self.path)
                raise
        else:
            LOG.info('Ownership of %s already %d:%d',
                     self.path,
                     uid,
                     gid)

    def chcon(self, context):
        # If dir returns whether to recusively set context
        try:
            try:
                selinux.lsetfilecon(self.path, context)
                LOG.info('Setting selinux context of %s to %s',
                     self.path, context)
                return True
            except OSError as e:
                if self.is_dir and e.errno == 95:
                    # Operation not supported, assume NFS mount and skip
                    LOG.info('Setting selinux context not supported for %s',
                             self.path)
                    return False
                else:
                    raise
        except Exception:
            LOG.exception('Could not set selinux context of %s to %s:',
                          self.path, context)
            raise


class NovaStatedirOwnershipManager(object):
    """Class to manipulate the ownership of the nova statedir (/var/lib/nova).

       The nova uid/gid differ on the host and container images. An upgrade
       that switches from host systemd services to docker requires a change in
       ownership. Previously this was a naive recursive chown, however this
       causes issues if nova instance are shared via an NFS mount: any open
       filehandles in qemu/libvirt fail with an I/O error (LP1778465).

       Instead the upgrade/FFU ansible tasks now lay down a marker file when
       stopping and disabling the host systemd services. We use this file to
       determine the host nova uid/gid. We then walk the tree and update any
       files that have the host uid/gid to the docker nova uid/gid. As files
       owned by root/qemu etc... are ignored this avoids the issues with open
       filehandles. The marker is removed once the tree has been walked.

       For subsequent runs, or for a new deployment, we simply ensure that the
       docker nova user/group owns all directories. This is required as the
       directories are created with root ownership in host_prep_tasks (the
       docker nova uid/gid is not known in this context).
    """
    def __init__(self, statedir, upgrade_marker='upgrade_marker',
                 nova_user='nova', secontext_marker='../_nova_secontext',
                 exclude_paths=None):
        self.statedir = statedir
        self.nova_user = nova_user

        self.upgrade_marker_path = os.path.join(statedir, upgrade_marker)
        self.secontext_marker_path = os.path.normpath(os.path.join(statedir, secontext_marker))
        self.upgrade = os.path.exists(self.upgrade_marker_path)

        self.exclude_paths = [self.upgrade_marker_path]
        if exclude_paths is not None:
            for p in exclude_paths:
                if not p.startswith(os.path.sep):
                    p = os.path.join(self.statedir, p)
                self.exclude_paths.append(p)

        self.target_uid, self.target_gid = self._get_nova_ids()
        self.previous_uid, self.previous_gid = self._get_previous_nova_ids()
        self.id_change = (self.target_uid, self.target_gid) != \
            (self.previous_uid, self.previous_gid)
        self.target_secontext = self._get_secontext()

    def _get_nova_ids(self):
        nova_uid, nova_gid = pwd.getpwnam(self.nova_user)[2:4]
        return nova_uid, nova_gid

    def _get_previous_nova_ids(self):
        if self.upgrade:
            statinfo = os.stat(self.upgrade_marker_path)
            return statinfo.st_uid, statinfo.st_gid
        else:
            return self._get_nova_ids()

    def _get_secontext(self):
        if os.path.exists(self.secontext_marker_path):
            return selinux.lgetfilecon(self.secontext_marker_path)[1]
        else:
            return None

    def _walk(self, top, chcon=True):
        for f in os.listdir(top):
            pathname = os.path.join(top, f)

            if pathname in self.exclude_paths:
                continue

            try:
                pathinfo = PathManager(pathname)
                LOG.info("Checking %s", pathinfo)
                if pathinfo.is_dir:
                    # Always chown the directories
                    pathinfo.chown(self.target_uid, self.target_gid)
                    chcon_r = chcon
                    if chcon:
                        chcon_r = pathinfo.chcon(self.target_secontext)
                    self._walk(pathname, chcon_r)
                elif self.id_change:
                    # Only chown files if it's an upgrade and the file is owned by
                    # the host nova uid/gid
                    pathinfo.chown(
                        self.target_uid if pathinfo.uid == self.previous_uid
                        else pathinfo.uid,
                        self.target_gid if pathinfo.gid == self.previous_gid
                        else pathinfo.gid
                    )
                    if chcon:
                        pathinfo.chcon(self.target_secontext)
            except Exception:
                # Likely to have been caused by external systems
                # interacting with this directory tree,
                # especially on NFS e.g snapshot dirs.
                # Just ignore it and continue on to the next entry
                continue

    def run(self):
        LOG.info('Applying nova statedir ownership')
        LOG.info('Target ownership for %s: %d:%d',
                 self.statedir,
                 self.target_uid,
                 self.target_gid)

        pathinfo = PathManager(self.statedir)
        LOG.info("Checking %s", pathinfo)
        pathinfo.chown(self.target_uid, self.target_gid)
        chcon = self.target_secontext is not None

        if chcon:
            pathinfo.chcon(self.target_secontext)

        self._walk(self.statedir, chcon)

        if self.upgrade:
            LOG.info('Removing upgrade_marker %s',
                     self.upgrade_marker_path)
            os.unlink(self.upgrade_marker_path)

        LOG.info('Nova statedir ownership complete')


def get_exclude_paths():
    exclude_paths = os.environ.get('NOVA_STATEDIR_OWNERSHIP_SKIP')
    if exclude_paths is not None:
        exclude_paths = exclude_paths.split(os.pathsep)
    return exclude_paths


if __name__ == '__main__':
    NovaStatedirOwnershipManager('/var/lib/nova', exclude_paths=get_exclude_paths()).run()



#!/usr/bin/python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os
import tempfile
import yaml

from oslotest import base

from tools import convert_heat_nic_config_to_ansible_j2 as converter

FAKE_PARAM_TO_VAR_MAP = {
    'ControlPlaneDefaultRoute': '{{ ctlplane_gateway_ip }}',
    'ControlPlaneIp': '{{ ctlplane_ip }}',
    'ControlPlaneMtu': '{{ ctlplane_mtu }}',
    'ControlPlaneStaticRoutes': '{{ ctlplane_host_routes }}',
    'ControlPlaneSubnetCidr': '{{ ctlplane_subnet_cidr }}',
    'DnsSearchDomains': '{{ dns_search_domains }}',
    'DnsServers': '{{ ctlplane_dns_nameservers }}',
    'InternalApiInterfaceDefaultRoute': '{{ internal_api_gateway_ip }}',
    'InternalApiInterfaceRoutes': '{{ internal_api_host_routes }}',
    'InternalApiIpSubnet': '{{ internal_api_ip }}/{{ internal_api_cidr }}',
    'InternalApiMtu': '{{ internal_api_mtu }}',
    'InternalApiNetworkVlanID': '{{ internal_api_vlan_id }}',
    'NumDpdkInterfaceRxQueues': '{{ num_dpdk_interface_rx_queues }}',
    'BondInterfaceOvsOptions': '{{ bond_interface_ovs_options }}',
    'TenantInterfaceDefaultRoute': '{{ tenant_gateway_ip }}',
    'TenantInterfaceRoutes': '{{ tenant_host_routes }}',
    'TenantIpSubnet': '{{ tenant_ip }}/{{ tenant_cidr }}',
    'TenantMtu': '{{ tenant_mtu }}',
    'TenantNetworkVlanID': '{{ tenant_vlan_id }}',
}


class ConvertToAnsibleJ2TestCase(base.BaseTestCase):

    def setUp(self):
        super(ConvertToAnsibleJ2TestCase, self).setUp()
        self.fake_param_to_var_map = FAKE_PARAM_TO_VAR_MAP
        stack_env_file = os.path.join(
            os.path.dirname(__file__),
            'nic_config_convert_samples/stack_env_simple.yaml')
        networks_file = os.path.join(
            os.path.dirname(__file__),
            'nic_config_convert_samples/networks_file_simple.yaml')
        with open(stack_env_file, 'r') as f:
            self.fake_stack_env = yaml.safe_load(f.read())
        self.convert = converter.ConvertToAnsibleJ2(self.fake_stack_env,
                                                    networks_file)

    def test_to_j2_var(self):
        self.assertEqual('{{ some_var }}',
                         converter.ConvertToAnsibleJ2.to_j2_var('some_var'))

    def test_to_j2_var_raises_on_unsupported_type(self):
        for _type in [list(), dict(), int(), bool()]:
            self.assertRaises(RuntimeError,
                              converter.ConvertToAnsibleJ2.to_j2_var,
                              _type)

    def test_create_param_to_var_map(self):
        networks_file = os.path.join(
            os.path.dirname(__file__),
            'nic_config_convert_samples/networks_file_simple.yaml')
        param_to_var_map = self.convert.create_param_to_var_map(networks_file)
        self.assertEqual(self.fake_param_to_var_map.keys(),
                         param_to_var_map.keys())
        for key in param_to_var_map:
            self.assertEqual(self.fake_param_to_var_map[key],
                             param_to_var_map[key])

    def test_convert_get_param(self):
        for k, v in self.fake_param_to_var_map.items():
            old = {'get_param': k}
            new = self.convert.convert_get_param(old)
            self.assertEqual(v, new)

        for k, v in self.fake_stack_env['parameter_defaults'].items():
            old = {'get_param': k}
            new = self.convert.convert_get_param(old)
            if isinstance(v, str):
                self.assertTrue(v in new)
                self.assertTrue(new.startswith(converter.QUOTE_FIX + v))
            elif isinstance(v, (bool, int, list, dict)):
                self.assertEqual(v, new)

    def test_convert_get_param_unable_param_not_defined(self):
        old = {'get_param': 'UNKNOWN_PARAM'}
        new = self.convert.convert_get_param(old)
        self.assertEqual(converter.QUOTE_FIX
                         + ('NEED MANUAL CONVERSION: {\'get_param\': '
                           '\'UNKNOWN_PARAM\'}') + converter.QUOTE_FIX, new)

    def test_convert_get_param_comlex_data_struct_not_supported(self):
        old = {'get_param': ['COMPLEX', 'DATA']}
        new = self.convert.convert_get_param(old)
        self.assertEqual(converter.QUOTE_FIX
                         + ('NEED MANUAL CONVERSION: {\'get_param\': '
                            '[\'COMPLEX\', \'DATA\']}') + converter.QUOTE_FIX,
                         new)

    def test_convert_get_attr(self):
        old = {'get_attr': ['MinViableMtu', 'value']}
        new = self.convert.convert_get_attr(old)
        self.assertEqual('{{ min_viable_mtu }}', new)

    def test_convert_get_attr_not_suported(self):
        old = {'get_attr': ['UNSUPPORTED', 'value']}
        new = self.convert.convert_get_attr(old)
        self.assertEqual(('NEED MANUAL CONVERSION: '
                          '{\'get_attr\': [\'UNSUPPORTED\', \'value\']}'), new)

    def test_convert_list_join(self):
        list_join_attrs = ['/', ['foo', 'bar']]
        new = self.convert.convert_list_join(list_join_attrs)
        self.assertEqual('foo/bar', new)

    def test_convert_list_concat(self):
        list_concat_attrs = [['a_list', 'list_a'], ['list_b', 'b_list']]
        new = self.convert.convert_list_concat(list_concat_attrs)
        self.assertEqual(
            ("{{ [['%_fix_quote_%a_list%_fix_quote_%', "
             "'%_fix_quote_%list_a%_fix_quote_%'], "
             "['%_fix_quote_%list_b%_fix_quote_%', "
             "'%_fix_quote_%b_list%_fix_quote_%']] "
             "| flatten }}"), new)

    def test_convert_list_concat_raise_if_not_list(self):
        list_concat_attrs = 'NOT_A_LIST'
        self.assertRaises(RuntimeError,
                          self.convert.convert_list_concat,
                          list_concat_attrs)

    def test_convert_list_concat_unique(self):
        convert_list_concat_unique = [['a_list', 'b_list'],
                                      ['list_b', 'b_list']]
        new = self.convert.convert_list_concat_unique(
            convert_list_concat_unique)
        self.assertEqual(
            ("{{ [['%_fix_quote_%a_list%_fix_quote_%', "
             "'%_fix_quote_%b_list%_fix_quote_%'], "
             "['%_fix_quote_%list_b%_fix_quote_%', "
             "'%_fix_quote_%b_list%_fix_quote_%']] "
             "| flatten | unique }}"), new)

    def test_convert_list_concat_unique_raise_if_not_list(self):
        convert_list_concat_unique = 'NOT_A_LIST'
        self.assertRaises(RuntimeError,
                          self.convert.convert_list_concat_unique,
                          convert_list_concat_unique)

    def test_recursive_convert_str_bool_int_not_converted(self):
        # old is a string
        self.assertEqual('string', self.convert.recursive_convert('string'))
        # old is a boolean
        self.assertEqual(True, self.convert.recursive_convert(True))
        # old is a number
        self.assertEqual(1, self.convert.recursive_convert(1))

    def test_recursive_convert_nothing_to_convert(self):
        old = {'foo': 'bar', 'baz': ['a', 'b', {'c': 'd'}]}
        self.assertEqual(old, self.convert.recursive_convert(old))

    def test_recursive_convert_complex(self):
        addresses = [
            {'ip_netmask': {
                'list_join': ['/',
                              [{'get_param': 'ControlPlaneIp'},
                               {'get_param': 'ControlPlaneSubnetCidr'}]
                              ]}}]
        old = {'type': 'interface', 'name': 'nic1', 'use_dhcp': 'false',
               'addresses': addresses}
        expected = {'name': 'nic1', 'type': 'interface', 'use_dhcp': 'false',
                    'addresses': [
                        {'ip_netmask':
                             '{{ ctlplane_ip }}/{{ ctlplane_subnet_cidr }}'}]}
        self.assertEqual(expected, self.convert.recursive_convert(old))

    def test_recursive_convert_unsupported_intrinsic_fn(self):
        for fn in converter.UNSUPPORTED_HEAT_INTRINSIC_FUNCTIONS:
            old = {'foo': 'bar', 'baz': ['a', 'b', {fn: 'd'}]}
            unsupported_str = (
                "UNSUPPORTED HEAT INTRINSIC FUNCTION {x} REQUIRES MANUAL "
                "CONVERSION {{'{x}': 'd'}}".format(x=fn))
            new = {'foo': 'bar', 'baz': ['a', 'b', unsupported_str]}
            self.assertEqual(new, self.convert.recursive_convert(old))

    def test_convert_template(self):
        template_file = os.path.join(
            os.path.dirname(__file__),
            'nic_config_convert_samples/heat_templates/simple.yaml')
        j2_config, mtu_header, j2_header = self.convert.convert_template(
            template_file)
        self.assertIsNone(mtu_header)
        self.assertIsNone(j2_header)
        expected = [
            {'name': 'nic1',
             'type': 'interface',
             'use_dhcp': False,
             'addresses': [{
                 'ip_netmask': '{{ ctlplane_ip }}/{{ ctlplane_subnet_cidr }}'}
             ]},
            {'name': 'nic2',
             'type': 'interface',
             'use_dhcp': False},
            {'type': 'vlan',
             'device': 'nic2',
             'vlan_id': '{{ internal_api_vlan_id }}',
             'addresses': [{
                 'ip_netmask': '{{ internal_api_ip }}/{{ internal_api_cidr }}'}
             ]},
            {'type': 'ovs_bridge',
             'name': 'bridge_name',
             'dns_servers': '{{ ctlplane_dns_nameservers }}',
             'members': [
                 {'type': 'interface',
                  'name': 'nic3',
                  'primary': True},
                 {'type': 'vlan',
                  'vlan_id': '{{ tenant_vlan_id }}',
                  'addresses': [{
                      'ip_netmask': '{{ tenant_ip }}/{{ tenant_cidr }}'}
                  ]},
             ]},
        ]
        self.assertEqual(expected, j2_config['network_config'])

    def convert_heat_to_ansible_j2(self, heat_template, j2_reference,
                                   networks_file='network_file_complex.yaml',
                                   stack_env='stack_env_simple.yaml'):
        networks_file = os.path.join(
            os.path.dirname(__file__),
            'nic_config_convert_samples/' + networks_file)
        template_file = os.path.join(
            os.path.dirname(__file__),
            'nic_config_convert_samples/heat_templates/' + heat_template)
        reference_file = os.path.join(
            os.path.dirname(__file__),
            'nic_config_convert_samples/j2_references/' + j2_reference)
        stack_env_file = os.path.join(
            os.path.dirname(__file__),
            'nic_config_convert_samples/' + stack_env)

        with open(stack_env_file, 'r') as f:
            stack_env = yaml.safe_load(f.read())
        convert = converter.ConvertToAnsibleJ2(stack_env, networks_file)
        j2_config, mtu_header, j2_header = convert.convert_template(
            template_file)

        with tempfile.TemporaryDirectory() as temp_dir:
            j2_template = os.path.abspath(temp_dir) + '/j2_template.j2'
            converter.write_j2_template(j2_template, j2_config, mtu_header,
                                        j2_header)
            with open(reference_file, 'r') as a:
                with open(j2_template, 'r') as b:
                    reference = a.read()
                    result = b.read()

        self.assertEqual(reference, result)

    def test_convert_and_write_file_simple(self):
        self.convert_heat_to_ansible_j2(
            'simple.yaml', 'simple.j2',
            networks_file='networks_file_simple.yaml')

    def test_convert_and_write_file_complex01_incomplete(self):
        self.convert_heat_to_ansible_j2('complex.yaml',
                                        'complex_incomplete.j2')

    def test_convert_and_write_file_complex01_complete(self):
        self.convert_heat_to_ansible_j2('complex.yaml', 'complex_complete.j2',
                                        stack_env='stack_env_complex.yaml')

    def test_convert_2_linux_bonds_vlan_controller(self):
        self.convert_heat_to_ansible_j2('2-linux-bonds-vlans-controller.yaml',
                                        '2-linux-bonds-vlans-controller.j2')

    def test_convert_bond_vlans_controller(self):
        self.convert_heat_to_ansible_j2('bond-vlans-controller.yaml',
                                        'bond-vlans-controller.j2')

    def test_convert_multiple_nics_vlans_controller(self):
        self.convert_heat_to_ansible_j2('multiple-nics-vlans-controller.yaml',
                                        'multiple-nics-vlans-controller.j2')

    def test_convert_single_nic_linux_bridge_vlans_controller(self):
        self.convert_heat_to_ansible_j2(
            'single-nic-linux-bridge-vlans-controller.yaml',
            'single-nic-linux-bridge-vlans-controller.j2')

    def test_convert_single_nic_vlans_controller(self):
        self.convert_heat_to_ansible_j2('single-nic-vlans-controller.yaml',
                                        'single-nic-vlans-controller.j2')



#!/usr/bin/python3
#
# Copyright 2019 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#

import argparse
import errno
import json
import os
import sys
import yaml
import yaql


def parse_opts(argv):
    parser = argparse.ArgumentParser(
        description='Render the Ansible tasks based in the role and the tags selected.'
                    'Those tasks can be used for debugging or linting purposes.')
    subp = parser.add_mutually_exclusive_group(required=True)

    parser.add_argument('--output', required=True, metavar='<tasks directory output>',
                        help="The folder to store the rendered tasks",
                        )

    parser.add_argument('--ansible-tasks', nargs="+", required=True,
                        metavar='<ansible tasks to be rendered>',
                        help='THT tags to filter the Ansible rendering '
                             'i.e. update_tasks')

    subp.add_argument('--roles-list', nargs="+", metavar='<list of roles to render>',
                      help='Composable roles to filter the Ansible rendering '
                           'i.e. Controller Compute')

    subp.add_argument('--all', action='store_true',
                      help='Process all services in the resource registry at once, '
                           'this allows to test all services templates avoiding '
                             'reading and generating all the files.')

    opts = parser.parse_args(argv[1:])
    return opts


def main():
    opts = parse_opts(sys.argv)
    engine = yaql.factory.YaqlFactory().create()
    output = opts.output
    # We open the resource registry once
    resource_registry = "./overcloud-resource-registry-puppet.yaml"
    resource_reg = yaml.safe_load(open(os.path.join(resource_registry), 'r'))

    if (opts.all):
        # This means we will parse all the services defined
        # by default in the resource registry
        roles_list = ["overcloud-resource-registry-puppet"]
    else:
        roles_list = opts.roles_list

    for role in roles_list:
        # We open the role file only once.
        if (opts.all):
            # The service definition will be the same resource registry
            role_resources = resource_reg
        else:
            role_resources = yaml.safe_load(
                open(os.path.join("./roles/", role + ".yaml"), 'r'))

        for section_task in opts.ansible_tasks:
            if(opts.all):
                # We get all the services in the resource_registry section
                expression = engine(
                    "$.resource_registry"
                )
            else:
                expression = engine(
                    "$.ServicesDefault.flatten().distinct()"
                )
            heat_resources = expression.evaluate(data=role_resources)
            role_ansible_tasks = []

            for resource in heat_resources:
                if(opts.all):
                    # If we use the resource registry as the source of the
                    # data we need to split the service name of the
                    # service config definition
                    resource = resource.split(' ')[0]
                expression = engine(
                  "$.resource_registry.get('" + resource + "')"
                )
                config_file = expression.evaluate(data=resource_reg)
                if(config_file is not None):
                    if('::' in config_file):
                        print("This is a nested Heat resource")
                    else:
                        data_source = yaml.safe_load(
                            open("./" + config_file, 'r'))
                        expression = engine(
                          "$.outputs.role_data.value.get(" + section_task + ").flatten().distinct()"
                        )
                        try:
                            ansible_tasks = expression.evaluate(data=data_source)
                            print(ansible_tasks)
                            role_ansible_tasks = role_ansible_tasks + ansible_tasks
                        except Exception:
                            print("There are no tasks in the configuration file")
            if (role_ansible_tasks != []):
                tasks_output_file = os.path.join(output, role + "_" + section_task + ".yml")
                if not os.path.exists(os.path.dirname(tasks_output_file)):
                    try:
                        os.makedirs(os.path.dirname(tasks_output_file))
                    except OSError as exc:
                        if exc.errno != errno.EEXIST:
                            raise
                save = open(tasks_output_file, 'w+')
                yaml.dump(yaml.safe_load(json.dumps(role_ansible_tasks)),
                          save, default_flow_style=False)


if __name__ == '__main__':
    main()



#!/usr/bin/python3

# Copyright 2023 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#

import argparse
import copy
import datetime
import os
import shutil
import sys
import yaml


from io import StringIO
from tripleo_common.image import kolla_builder
from tripleoclient import utils


def parse_opts(argv):
    parser = argparse.ArgumentParser(
        description='Tool to help to create the multi-rhel container image file')
    parser.add_argument(
        "--output-env-file",
        dest="output_env_file",
        metavar='<file path>',
        required=True,
        help="File to write environment file containing default "
             "ContainerImagePrepare value.",
    )
    parser.add_argument(
        '--local-push-destination',
        dest='push_destination',
        action='store_true',
        default=False,
        help='Include a push_destination to trigger upload to a local '
             'registry.'
    )
    parser.add_argument(
        '--enable-registry-login',
        dest='registry_login',
        action='store_true',
        default=False,
        help='Use this flag to enable the flag to have systems attempt '
               'to login to a remote registry prior to pulling their '
               'containers. This flag should be used when '
               '--local-push-destination is *NOT* used and the target '
               'systems will have network connectivity to the remote '
               'registries. Do not use this for an overcloud that '
               'may not have network connectivity to a remote registry.'
    )
    parser.add_argument(
        '--enable-multi-rhel',
        dest='multi_rhel',
        action='store_true',
        default=False,
        help='Use this flag to enable multi-rhel'
    )
    parser.add_argument(
        '--excludes',
        dest='excludes',
        action='append',
        default=[],
        help='List of services to include/exclude'
    )
    parser.add_argument(
        '--major-override',
        dest='major',
        action='store',
        default='{}',
        help='The override parameters for major release'
    )
    parser.add_argument(
        '--minor-override',
        dest='minor',
        action='store',
        default='{}',
        help='The override parameters for minor release'
    )
    parser.add_argument(
        '--role',
        dest='roles',
        action='append',
        default=[],
        help='List of roles'
    )
    parser.add_argument(
        '--role-file',
        dest='rolefile',
        action='store',
        default='',
        help='role_data.yaml file'
    )

    opts = parser.parse_args(argv[1:])
    return opts


def build_env_file(params):
    f = StringIO()
    f.write('# Generated with the following on %s\n#\n' %
            datetime.datetime.now().isoformat())
    yaml.safe_dump({'parameter_defaults': params}, f,
                   default_flow_style=False,
                   sort_keys=False)
    return f.getvalue()


parsed_args = parse_opts(sys.argv)

auth_required = False

cip = copy.deepcopy(kolla_builder.CONTAINER_IMAGE_PREPARE_PARAM)
if parsed_args.push_destination:
    for entry in cip:
        entry['push_destination'] = True
params = {
    'ContainerImagePrepare': cip
}
if parsed_args.registry_login:
    if parsed_args.push_destination:
        print('[WARNING] --local-push-destination was used '
                         'with --enable-registry-login. Please make '
                         'sure you understand the use of these '
                         'parameters together as they can cause '
                         'deployment failures.')
    print('[NOTE] Make sure to update the paramter_defaults'
          ' with ContainerImageRegistryCredentials for the '
          'registries requiring authentication.')
    params['ContainerImageRegistryLogin'] = True

if parsed_args.multi_rhel:
    cip_exc = copy.deepcopy(cip)
    cip_inc = copy.deepcopy(cip)
    # set default ceph image to 5 always, otherwise cephv6 images will be considered
    # as default from CONTAINER_IMAGE_PREPARE_PARAM
    cip_inc[0]['set']["ceph_image"] = "rhceph-5-rhel8"
    cip_exc[0]['set']["ceph_image"] = "rhceph-5-rhel8"
    params['ContainerImagePrepare'][0]['set']["ceph_image"] = "rhceph-5-rhel8"
    if parsed_args.major is not None:
        major = yaml.safe_load(parsed_args.major)
    if len(parsed_args.excludes) > 0:
        cip_exc[0]['excludes'] = copy.deepcopy(parsed_args.excludes)
        cip_inc[0]['includes'] = copy.deepcopy(parsed_args.excludes)
        if parsed_args.minor is not None:
            minor = yaml.safe_load(parsed_args.minor)
            for key in minor.keys():
                if key in cip_inc[0]['set'].keys():
                    cip_inc[0]['set'][key] = minor[key]
        if parsed_args.major is not None:
            major = yaml.safe_load(parsed_args.major)
            for key in major.keys():
                if key in cip_exc[0]['set'].keys():
                    cip_exc[0]['set'][key] = major[key]
                params_set = params['ContainerImagePrepare'][0]['set']
                if key in params_set.keys():
                    params_set[key] = major[key]
    base_role = [cip_exc[0], cip_inc[0]]
    if parsed_args.rolefile != '':
        read_roles = []
        if os.path.exists(parsed_args.rolefile):
            with open(parsed_args.rolefile) as file:
                roles_f = yaml.safe_load(file)
                for role in roles_f:
                    read_roles.append(role['name'])
        else:
            print('[ERROR] {} role file does'
                           ' not exits'.format(parsed_args.rolefile))
        roles = read_roles
    else:
        roles = parsed_args.roles
    params['MultiRhelRoleContainerImagePrepare'] = base_role
    for role in roles:
        params[('{}ContainerImagePrepare').format(role)] = base_role
env_data = build_env_file(params)
if parsed_args.output_env_file:
    if os.path.exists(parsed_args.output_env_file):
        print("Output env file exists, "
                         "moving it to backup.")
        shutil.move(parsed_args.output_env_file,
                    parsed_args.output_env_file + ".backup")
    utils.safe_write(parsed_args.output_env_file, env_data)



#!/usr/bin/python3
#
# Copyright 2022 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from configparser import ConfigParser
import logging
import os
import subprocess
import sys
from urllib import parse as urlparse

config = ConfigParser(strict=False)

debug = os.getenv('__OS_DEBUG', 'false')

if debug.lower() == 'true':
    loglevel = logging.DEBUG
else:
    loglevel = logging.INFO

logging.basicConfig(stream=sys.stdout, level=loglevel)
LOG = logging.getLogger('nova_api_ensure_default_cells')

NOVA_CFG = '/etc/nova/nova.conf'
CELL0_ID = '00000000-0000-0000-0000-000000000000'
DEFAULT_CELL_NAME = 'default'


def template_netloc_credentials(netloc, index=None):
    if '@' in netloc:
        userpass, hostport = netloc.split('@', 1)
        has_pass = ':' in userpass
        if index is None:
            cred_template = '{username}'
            if has_pass:
                cred_template += ':{password}'
        else:
            cred_template = '{{username{index}}}'.format(index=index)
            if has_pass:
                cred_template += ':{{password{index}}}'.format(index=index)
        return '@'.join((cred_template, hostport))
    else:
        return netloc


def template_url(url):
    parsed = urlparse.urlparse(url)
    if ',' in parsed.netloc:
        orig_netlocs = parsed.netloc.split(',')
        templ_netlocs = []
        index = 0
        for netloc in orig_netlocs:
            index += 1
            templ_netlocs.append(template_netloc_credentials(netloc, index))
        new_netloc = ','.join(templ_netlocs)
    else:
        new_netloc = template_netloc_credentials(parsed.netloc)
    return parsed._replace(netloc=new_netloc).geturl()


def parse_list_cells(list_cells_output):
    list_cells_lines = list_cells_output.split('\n')
    if len(list_cells_lines) < 5:
        raise ValueError('Invalid nova-manage cell_v2 list_cells output')

    data_rows = list_cells_lines[3:-2]
    by_name = {}
    by_uuid = {}

    for row in data_rows:
        parts = row.split('|')
        entry = {
            'name': parts[1].strip(),
            'uuid': parts[2].strip(),
            'transport_url': parts[3].strip(),
            'database_connection': parts[4].strip(),
        }
        by_name[entry['name']] = entry
        by_uuid[entry['uuid']] = entry

    return by_name, by_uuid


def create_or_update_default_cells(cell0_db, default_db, default_transport_url):
    list_cells_cmd = ['/usr/bin/nova-manage', 'cell_v2', 'list_cells', '--verbose']
    list_cells_output = subprocess.check_output(list_cells_cmd, encoding='utf-8')
    cells_by_name, cells_by_uuid = parse_list_cells(list_cells_output)

    if CELL0_ID in cells_by_uuid:
        LOG.info('Setting cell0 database connection to \'{}\''.format(cell0_db))
        cmd = [
            '/usr/bin/nova-manage', 'cell_v2', 'update_cell',
            '--cell_uuid', CELL0_ID,
            '--database_connection', cell0_db,
            '--transport-url', 'none:///'
        ]
    else:
        LOG.info('Creating cell0 with database connection \'{}\''.format(cell0_db))
        cmd = [
            '/usr/bin/nova-manage', 'cell_v2', 'map_cell0',
             '--database_connection', cell0_db
        ]
    subprocess.check_call(cmd)

    if DEFAULT_CELL_NAME in cells_by_name:
        LOG.info('Setting default cell database connection to \'{}\' and transport url to \'{}\''.format(
            default_db, default_transport_url))
        cmd = [
            '/usr/bin/nova-manage', 'cell_v2', 'update_cell',
            '--cell_uuid', cells_by_name[DEFAULT_CELL_NAME]['uuid'],
            '--database_connection', default_db,
            '--transport-url', default_transport_url
        ]
    else:
        LOG.info('Creating default cell with database connection \'{}\' and transport url \'{}\''.format(
            default_db, default_transport_url))
        cmd = [
            '/usr/bin/nova-manage', 'cell_v2', 'create_cell',
            '--name', DEFAULT_CELL_NAME,
             '--database_connection', default_db,
            '--transport-url', default_transport_url
        ]
    subprocess.check_call(cmd)


def replace_db_name(db_url, db_name):
    return urlparse.urlparse(db_url)._replace(path=db_name).geturl()


if __name__ == '__main__':
    if os.path.isfile(NOVA_CFG):
        try:
            config.read(NOVA_CFG)
        except Exception:
            LOG.exception('Error while reading nova.conf:')
            sys.exit(1)
    else:
        LOG.error('Nova configuration file %s does not exist', NOVA_CFG)
        sys.exit(1)

    default_database_connection = config.get('database', 'connection')
    cell0_database_connection = replace_db_name(default_database_connection, 'nova_cell0')
    default_transport_url = config.get('DEFAULT', 'transport_url')

    create_or_update_default_cells(
        template_url(cell0_database_connection),
        template_url(default_database_connection),
        template_url(default_transport_url)
    )



#!/usr/bin/python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import argparse
import os
import yaml


def parse_opts():
    """Parse all of the conversion options."""

    parser = argparse.ArgumentParser(
        description="Convert a network V1 template to a V2 template."
    )
    parser.add_argument(
        "v1", metavar="<network_data.yaml>", help="Existing V1 Template."
    )

    return parser.parse_args()


def main():
    """Convert a network v1 template to the network v2 format.

    The V1 template will be converted to V2 format. The V1 template will be
    saved as a backup file before writing the V2 net-data format.
    """

    args = parse_opts()
    net_data_file = os.path.abspath(os.path.expanduser(args.v1))
    with open(net_data_file) as f:
        template_data = yaml.safe_load(f)

    new_template_data = list()
    for item in template_data:
        new_item = dict()
        item.pop("enabled", False)  # Drop unused var
        name = new_item["name"] = item.pop("name")
        name_lower = new_item["name_lower"] = item.pop(
            "name_lower", name.lower()
        )
        new_item["vip"] = item.pop("vip", False)
        new_item["mtu"] = item.pop("mtu", 1500)
        new_item["ipv6"] = item.pop("ipv6", False)
        new_item["subnets"] = item.pop("subnets", dict())
        new_item["subnets"]["{}_subnet".format(name_lower)] = item
        new_template_data.append(new_item)

    os.rename(net_data_file, "{}.bak".format(net_data_file))
    try:
        # content is converted to yaml before opening the file.
        # This is done to ensure that we're not breaking any existing files
        # during the conversion process.
        dump_yaml = yaml.safe_dump(
            new_template_data, default_style=False, sort_keys=False
        )
    except Exception as e:
        print("Conversion could not be completed. Error:{}".format(str(e)))
    else:
        with open(net_data_file, "w") as f:
            f.write(dump_yaml)


if __name__ == "__main__":
    main()



#!/usr/bin/python3
#
# Copyright 2018 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# shell script to check if placement API is up after X attempts.
# Default max is 60 iterations with 10s (default) timeout in between.

import logging
import os
import re
import six
import sys
import time

from keystoneauth1.identity import v3
from keystoneauth1 import session
from keystoneclient.v3 import client
import requests

# In python3 SafeConfigParser was renamed to ConfigParser and the default
# for duplicate options default to true. In case of nova it is valid to
# have duplicate option lines, e.g. passthrough_whitelist which leads to
# issues reading the nova.conf
# https://bugs.launchpad.net/tripleo/+bug/1827775
if six.PY3:
    from six.moves.configparser import ConfigParser
    config = ConfigParser(strict=False)
else:
    from six.moves.configparser import SafeConfigParser
    config = SafeConfigParser()


debug = os.getenv('__OS_DEBUG', 'false')

if debug.lower() == 'true':
    loglevel = logging.DEBUG
else:
    loglevel = logging.INFO

logging.basicConfig(stream=sys.stdout, level=loglevel)
LOG = logging.getLogger('placement_wait_for_service')

iterations = 60
timeout = 10
placement_cfg = '/etc/placement/placement.conf'

if __name__ == '__main__':
    if os.path.isfile(placement_cfg):
        try:
            config.read(placement_cfg)
        except Exception:
            LOG.exception('Error while reading placement.conf:')
    else:
        LOG.error('Placement configuration file %s does not exist',
                  placement_cfg)
        sys.exit(1)

    # get keystone client with details from [keystone_authtoken] section
    auth = v3.Password(
        user_domain_name=config.get('keystone_authtoken', 'user_domain_name'),
        username=config.get('keystone_authtoken', 'username'),
        password=config.get('keystone_authtoken', 'password'),
        project_name=config.get('keystone_authtoken', 'project_name'),
        project_domain_name=config.get('keystone_authtoken',
                                       'project_domain_name'),
        auth_url=config.get('keystone_authtoken', 'auth_url') + '/v3')
    sess = session.Session(auth=auth, verify=False)
    keystone = client.Client(session=sess, interface='internal')

    iterations_endpoint = iterations
    placement_endpoint_url = None
    while iterations_endpoint > 1:
        iterations_endpoint -= 1
        try:
            # get placement service id
            placement_service_id = keystone.services.list(
                name='placement')[0].id

            # get placement endpoint
            # Note: puppet-placement does not support setting the interface
            #       until we have https://review.opendev.org/688862.
            #       Lets hard code 'internal' for now.
            placement_endpoint_url = keystone.endpoints.list(
                service=placement_service_id,
                region=config.get('keystone_authtoken', 'region_name'),
                interface='internal')[0].url
            if not placement_endpoint_url:
                LOG.error('Failed to get placement service endpoint!')
            else:
                break
        except Exception:
            LOG.exception('Retry - Failed to get placement service endpoint:')
        time.sleep(timeout)

    if not placement_endpoint_url:
        LOG.error('Failed to get placement service endpoint!')
        sys.exit(1)

    # we should have CURRENT in the request response from placement:
    # {"versions": [{"status": "CURRENT", "min_version": "1.0", "max_version":
    # "1.29", "id": "v1.0", "links": [{"href": "", "rel": "self"}]}]}
    response_reg = re.compile('.*CURRENT,*')

    while iterations > 1:
        iterations -= 1
        try:
            r = requests.get(placement_endpoint_url + '/', verify=False)
            if r.status_code == 200 and response_reg.match(r.text):
                LOG.info('Placement service up! - %s', r.text)
                sys.exit(0)
                break
            else:
                LOG.info('response - %r', r)
                LOG.info('Placement service not up - %s, %s',
                         r.status_code,
                         r.text)
        except Exception:
            LOG.exception('Error query the placement endpoint:')
        time.sleep(timeout)

    sys.exit(1)

# vim: set et ts=4 sw=4 :



#!/usr/bin/python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
Generate the endpoint_map.yaml template from data in the endpoint_data.yaml
file.

By default the files in the same directory as this script are operated on, but
different files can be optionally specified on the command line.

The --check option verifies that the current output file is up-to-date with the
latest data in the input file. The script exits with status code 2 if a
mismatch is detected.
"""
import collections
import copy
import itertools
import os
import sys
import yaml


__all__ = ['load_endpoint_data', 'generate_endpoint_map_template',
           'write_template', 'build_endpoint_map', 'check_up_to_date']

(IN_FILE, OUT_FILE) = ('endpoint_data.yaml', 'endpoint_map.yaml')

SUBST = (SUBST_IP_ADDRESS, SUBST_CLOUDNAME) = ('IP_ADDRESS', 'CLOUDNAME')
PARAMS = (PARAM_CLOUD_ENDPOINTS, PARAM_ENDPOINTMAP, PARAM_NETIPMAP,
          PARAM_SERVICENETMAP) = (
          'CloudEndpoints', 'EndpointMap', 'NetIpMap', 'ServiceNetMap')
FIELDS = (F_PORT, F_PROTOCOL, F_HOST) = ('port', 'protocol', 'host')

ENDPOINT_TYPES = frozenset(['Internal', 'Public', 'Admin'])


def get_file(default_fn, override=None, writable=False):
    if override == '-':
        if writable:
            return sys.stdout
        else:
            return sys.stdin

    if override is not None:
        filename = override
    else:
        filename = os.path.join(os.path.dirname(__file__), default_fn)

    return open(filename, 'w' if writable else 'r')


def load_endpoint_data(infile=None):
    with get_file(IN_FILE, infile) as f:
        return yaml.safe_load(f)


def net_param_name(endpoint_type_defn):
    return endpoint_type_defn['net_param'] + 'Network'


def endpoint_map_default(config):
    def map_item(ep_name, ep_type, svc):
        values = collections.OrderedDict([
            (F_PROTOCOL, str(svc[ep_type].get(F_PROTOCOL,
                                              svc.get(F_PROTOCOL, 'http')))),
            (F_PORT, str(svc[ep_type].get(F_PORT, svc[F_PORT]))),
            (F_HOST, SUBST_IP_ADDRESS),
        ])
        return ep_name + ep_type, values

    return collections.OrderedDict(map_item(ep_name, ep_type, svc)
                                   for ep_name, svc in sorted(config.items())
                                   for ep_type in sorted(set(svc) &
                                                         ENDPOINT_TYPES))


def make_parameter(ptype, default, description=None):
    param = collections.OrderedDict([('type', ptype), ('default', default)])
    if description is not None:
        param['description'] = description
    return param


def template_parameters(config):
    params = collections.OrderedDict()
    params[PARAM_NETIPMAP] = make_parameter('json', {}, 'The Net IP map')
    params[PARAM_SERVICENETMAP] = make_parameter('json', {},
                                                 'The Service Net map')
    params[PARAM_ENDPOINTMAP] = make_parameter('json',
                                               endpoint_map_default(config),
                                               'Mapping of service endpoint '
                                               '-> protocol. Typically set '
                                               'via parameter_defaults in the '
                                               'resource registry.')

    params[PARAM_CLOUD_ENDPOINTS] = make_parameter(
        'json',
        {},
        ('A map containing the DNS names for the different endpoints '
         '(external, internal_api, etc.)'))
    return params


def template_output_definition(endpoint_name,
                               endpoint_variant,
                               endpoint_type,
                               net_param,
                               uri_suffix=None,
                               name_override=None):
    def extract_field(field):
        assert field in FIELDS
        return {'get_param': ['EndpointMap',
                              endpoint_name + endpoint_type,
                              copy.copy(field)]}

    port = extract_field(F_PORT)
    protocol = extract_field(F_PROTOCOL)
    host_nobrackets = {
        'str_replace': collections.OrderedDict([
            ('template', extract_field(F_HOST)),
            ('params', {
                SUBST_IP_ADDRESS: {'get_param':
                                   ['NetIpMap',
                                    {'get_param': ['ServiceNetMap',
                                     net_param]}]},
                SUBST_CLOUDNAME: {'get_param':
                                  [PARAM_CLOUD_ENDPOINTS,
                                   {'get_param': ['ServiceNetMap',
                                     net_param]}]},
            })
        ])
    }
    host = {
        'str_replace': collections.OrderedDict([
            ('template', extract_field(F_HOST)),
            ('params', {
                SUBST_IP_ADDRESS: {'get_param':
                                   ['NetIpMap',
                                    {'str_replace':
                                    {'template': 'NETWORK_uri',
                                     'params': {'NETWORK':
                                     {'get_param': ['ServiceNetMap',
                                                    net_param]}}}}]},
                SUBST_CLOUDNAME: {'get_param':
                                  [PARAM_CLOUD_ENDPOINTS,
                                   {'get_param': ['ServiceNetMap',
                                     net_param]}]},
            })
        ])
    }
    uri_no_path = {
        'make_url': collections.OrderedDict([
            ('scheme', protocol),
            ('host', copy.deepcopy(host)),
            ('port', port)
        ])
    }
    uri_with_path = copy.deepcopy(uri_no_path)
    if uri_suffix is not None:
        path, pc, suffix = uri_suffix.partition('%')
        uri_with_path['make_url']['path'] = path
        if pc:
            uri_with_path = {'list_join': ['', [uri_with_path, pc + suffix]]}

    name = name_override if name_override is not None else (endpoint_name +
                                                            endpoint_variant +
                                                            endpoint_type)

    return name, {
        'host_nobrackets': host_nobrackets,
        'host': host,
        'port': extract_field('port'),
        'protocol': extract_field('protocol'),
        'uri': uri_with_path,
        'uri_no_suffix': uri_no_path,
    }


def template_endpoint_items(config):
    def get_svc_endpoints(ep_name, svc):
        for ep_type in set(svc) & ENDPOINT_TYPES:
            defn = svc[ep_type]
            for variant, suffix in defn.get('uri_suffixes',
                                            {'': None}).items():
                name_override = defn.get('names', {}).get(variant)
                yield template_output_definition(ep_name, variant, ep_type,
                                                 net_param_name(defn),
                                                 suffix,
                                                 name_override)
    return itertools.chain.from_iterable(sorted(get_svc_endpoints(ep_name,
                                                                  svc))
                                         for (ep_name,
                                              svc) in sorted(config.items()))


def generate_endpoint_map_template(config):
    return collections.OrderedDict([
        ('heat_template_version', 'wallaby'),
        ('description', 'A map of OpenStack endpoints. Since the endpoints '
         'are URLs, we need to have brackets around IPv6 IP addresses. The '
         'inputs to these parameters come from net_ip_uri_map, which will '
         'include these brackets in IPv6 addresses.'),
        ('parameters', template_parameters(config)),
        ('outputs', {
            'endpoint_map': {
                'value':
                    collections.OrderedDict(template_endpoint_items(config))
            }
        }),
    ])


autogen_warning = """### DO NOT MODIFY THIS FILE
### This file is automatically generated from endpoint_data.yaml
### by the script build_endpoint_map.py

"""


class TemplateDumper(yaml.SafeDumper):
    def represent_ordered_dict(self, data):
        return self.represent_dict(data.items())


TemplateDumper.add_representer(collections.OrderedDict,
                               TemplateDumper.represent_ordered_dict)


def write_template(template, filename=None):
    with get_file(OUT_FILE, filename, writable=True) as f:
        f.write(autogen_warning)
        yaml.dump(template, f, TemplateDumper, width=68)


def read_template(template, filename=None):
    with get_file(OUT_FILE, filename) as f:
        return yaml.safe_load(f)


def build_endpoint_map(output_filename=None, input_filename=None):
    if output_filename is not None and output_filename == input_filename:
        raise Exception('Cannot read from and write to the same file')
    config = load_endpoint_data(input_filename)
    template = generate_endpoint_map_template(config)
    write_template(template, output_filename)


def check_up_to_date(output_filename=None, input_filename=None):
    if output_filename is not None and output_filename == input_filename:
        raise Exception('Input and output filenames must be different')
    config = load_endpoint_data(input_filename)
    template = generate_endpoint_map_template(config)
    existing_template = read_template(output_filename)
    return existing_template == template


def get_options():
    import argparse

    parser = argparse.ArgumentParser(
        usage="%(prog)s [-i INPUT_FILE] [-o OUTPUT_FILE] [--check]",
        description=__doc__)
    parser.add_argument('-i', '--input', dest='input_file', action='store',
                        default=None,
                        help='Specify a different endpoint data file')
    parser.add_argument('-o', '--output', dest='output_file', action='store',
                        default=None,
                        help='Specify a different endpoint map template file')
    parser.add_argument('-c', '--check', dest='check', action='store_true',
                        default=False, help='Check that the output file is '
                                            'up to date with the data')
    parser.add_argument('-d', '--debug', dest='debug', action='store_true',
                        default=False, help='Print stack traces on error')

    return parser.parse_args()


def main():
    args = get_options()

    try:
        if args.check:
            if not check_up_to_date(args.output_file, args.input_file):
                print('EndpointMap template does not match input data. Please '
                      'run the build_endpoint_map.py tool to update the '
                      'template.', file=sys.stderr)
                sys.exit(2)
        else:
            build_endpoint_map(args.output_file, args.input_file)
    except Exception as exc:
        if args.debug:
            raise
        print('%s: %s' % (type(exc).__name__, str(exc)), file=sys.stderr)
        sys.exit(1)


if __name__ == '__main__':
    main()



# Copyright 2023 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#

import argparse
import copy
import sys
import yaml


def parse_opts(argv):
    parser = argparse.ArgumentParser(
        description='Help to modify the baremental deployment script')
    parser.add_argument('--baremetal-deployment', metavar='<baremetal_file>',
                        help="Path to baremetal-deployment.yaml")
    parser.add_argument('--src-role', metavar='<src_role>',
                        help='role')
    parser.add_argument('--dst-role', metavar='<dst_role>',
                        help='role')
    parser.add_argument('nodes', nargs="+", metavar='<node>',
                        help='List of nodes')
    opts = parser.parse_args(argv[1:])

    return opts


opts = parse_opts(sys.argv)
print("File {} src role {} dst role {} "
      "and nodes {}".format(opts.baremetal_deployment,
                            opts.src_role,
                            opts.dst_role,
                            opts.nodes))

with open(opts.baremetal_deployment) as file:
    baremetal = yaml.safe_load(file.read())
    role_src = None
    role_dst = None
    for role in baremetal:
        if role['name'] == opts.src_role:
            role_src = role
        else:
            if role['name'] == opts.dst_role:
                role_dst = role
    if role_src is not None:
        if role_dst is None:
            role_dst = copy.copy(role_src)
            role_dst['count'] = 0
            role_dst['instances'] = []
            role_dst['name'] = opts.dst_role
            hostname_format = "{}-%index%".format(role_dst['name'].lower())
            role_dst['hostname_format'] = hostname_format
            baremetal.append(role_dst)
    for node2move in opts.nodes:
        for node in role_src['instances']:
            if (node['hostname'] == node2move):
                role_dst['count'] = role_dst['count'] + 1
                role_dst['instances'].append(copy.copy(node))
                node['provisioned'] = False
                role_src['count'] = role_src['count'] - 1

if role_src is not None:
    with open(opts.baremetal_deployment, "w") as file:
        yaml.dump(baremetal, file)



#!/usr/bin/python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import argparse
import os

import ruamel.yaml
from ruamel.yaml import YAML

# Not all policy variables across services in THT are consistent. This mapping
# assoicates the service name to the right THT variable.
_SERVICE_MAP = {
    'barbican': 'BarbicanPolicies',
    'cinder': 'CinderApiPolicies',
    'designate': 'DesignateApiPolicies',
    'glance': 'GlanceApiPolicies',
    'ironic': 'IronicApiPolicies',
    'keystone': 'KeystonePolicies',
    'manila': 'ManilaApiPolicies',
    'neutron': 'NeutronApiPolicies',
    'nova': 'NovaApiPolicies',
    'octavia': 'OctaviaApiPolicies',
    'placement': 'PlacementPolicies'
}
_SCALAR = ruamel.yaml.scalarstring.DoubleQuotedScalarString

parser = argparse.ArgumentParser()
parser.add_argument(
    '-d', '--policy-dir', required=True,
    help=(
        'Directory containing policy.yaml files for OpenStack services. '
        'This script expects files to be named $SERVICE.yaml. For example '
        'nova.yaml for nova\'s policies.'
    )
)
args = parser.parse_args()

heat_template = {'parameter_defaults': {'EnforceSecureRbac': False}}
for filename in os.listdir(args.policy_dir):
    service = filename.split('.')[0]
    tht_var_name = _SERVICE_MAP.get(service)
    filepath = os.path.join(args.policy_dir, filename)
    with open(filepath, 'r') as f:
        safe_handler = YAML(typ='safe')
        # A lot of policy files have duplicate keys, which violates YAML. Allow
        # duplicate keys for the time being.
        safe_handler.allow_duplicate_keys = True
        policies = safe_handler.load(f)

    template = {}
    for name, value in policies.items():
        rule = name.split(':')[-1]
        rule = name.replace(':', '_')
        key = service + '-' + rule
        template[key] = {'key': _SCALAR(name), 'value': _SCALAR(value)}
    heat_template['parameter_defaults'][tht_var_name] = template

print(
    ruamel.yaml.dump(
        heat_template, Dumper=ruamel.yaml.RoundTripDumper, width=500
    )
)
